/*
 * Copyright (C) 2006 Samuel Weinig (sam.weinig@gmail.com)
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "FrameAG.h"

#include "TextResourceDecoder.h"
#include "Document.h"
#include "FrameLoadRequest.h"
#include "FramePrivate.h"
#include "Settings.h"
#include "PlatformKeyboardEvent.h"
#include "Plugin.h"
#include "RenderFrame.h"
#include "ResourceHandle.h"

namespace WebCore {

FrameAG::FrameAG(Page* page, Element* ownerElement,  PassRefPtr<EditorClient> editorClient, FrameAGClient* client)
    : Frame(page, ownerElement, editorClient)
    , m_client(client)
{
    Settings* settings = new Settings();
    settings->setAutoLoadImages(true);
    settings->setMediumFixedFontSize(13);
    settings->setMediumFontSize(16);
    settings->setSerifFontName("Times New Roman");
    settings->setFixedFontName("Courier New");
    settings->setSansSerifFontName("Arial");
    settings->setStdFontName("Times New Roman");
	settings->setStdEncoding("ISO-8859-1");
    settings->setIsJavaScriptEnabled(true);
    setSettings(settings);
}

FrameAG::~FrameAG()
{
    setView(0);
    clearRecordedFormValues();    
}

void FrameAG::urlSelected(const FrameLoadRequest& request, const Event* /*triggeringEvent*/)
{
    if (m_client)
        m_client->openURL(request.resourceRequest().url().url(), request.lockHistory());
}

void FrameAG::submitForm(const FrameLoadRequest& request)
{
    // FIXME: this is a hack inherited from FrameMac, and should be pushed into Frame
    const ResourceRequest& resourceRequest = request.resourceRequest();
    if (d->m_submittedFormURL == resourceRequest.url())
        return;
    d->m_submittedFormURL = resourceRequest.url();

    if (m_client)
        m_client->submitForm(resourceRequest.httpMethod(), resourceRequest.url(), &resourceRequest.httpBody());

    clearRecordedFormValues();
}

String FrameAG::userAgent() const
{
	return "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; SV1)";
}

void FrameAG::runJavaScriptAlert(String const& message)
{
	//NOT IMPLEMENTED
	return;
}

bool FrameAG::runJavaScriptConfirm(String const& message)
{
	//NOT IMPLEMENTED
	return false;
}

// FIXME: This needs to be unified with the keyPress method on FrameMac
bool FrameAG::keyPress(const PlatformKeyboardEvent& keyEvent)
{
	//NOT IMPLEMENTED
	return false;
}

void FrameAG::setTitle(const String &title)
{
    String text = title;
    text.replace('\\', backslashAsCurrencySymbol());

    m_client->setTitle(text);
}

void FrameAG::setStatusBarText(const String& status)
{
    String text = status;
    text.replace('\\', backslashAsCurrencySymbol());

    m_client->setStatusText(text);
}

void FrameAG::createNewWindow(const FrameLoadRequest& request, const WindowFeatures& features, Frame*& newFrame)
{
    m_client->createNewWindow(request.resourceRequest(), features, newFrame);
}

} // namespace WebCore
