/*
 * Copyright (C) 2006 Samuel Weinig (sam.weinig@gmail.com)
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef FrameAG_H
#define FrameAG_H

#include "Frame.h"

namespace WebCore {

class FrameLoadRequest;
struct WindowFeatures;

class FrameAGClient {
public:
    virtual void createNewWindow(const ResourceRequest&, const WindowFeatures&, Frame*& newFrame) = 0;
    virtual void openURL(const DeprecatedString&, bool lockHistory) = 0;
    virtual void submitForm(const String& method, const KURL&, const FormData*) = 0;
    virtual void setTitle(const String& title) = 0;
    virtual void setStatusText(const String& statusText) = 0;
	virtual ~FrameAGClient() { }
};

class FrameAG : public Frame {
public:
    FrameAG(Page*, Element*,  PassRefPtr<EditorClient>, FrameAGClient*);
    ~FrameAG();

    virtual void createNewWindow(const FrameLoadRequest&, const WindowFeatures&, Frame*& part);
    virtual void submitForm(const FrameLoadRequest&);
    virtual void urlSelected(const FrameLoadRequest&, const Event* triggeringEvent);

    virtual void setTitle(const String&);
    virtual void setStatusBarText(const String&);

    virtual ObjectContentType objectContentType(const KURL& url, const String& mimeType);
    virtual Plugin* createPlugin(Element*, const KURL&, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType);
    virtual Frame* createFrame(const KURL&, const String& name, Element* ownerElement, const String& referrer);
    virtual Widget* createJavaAppletWidget(const IntSize&, Element*, const HashMap<String, String>& args);

    virtual void scheduleClose();

    virtual void focusWindow();
    virtual void unfocusWindow();
    
    virtual void saveDocumentState();
    virtual void restoreDocumentState();
    
    virtual void addMessageToConsole(const String& message,  unsigned int lineNumber, const String& sourceID);

    virtual void runJavaScriptAlert(const String& message);
    virtual bool runJavaScriptConfirm(const String& message);
    virtual bool runJavaScriptPrompt(const String& message, const String& defaultValue, String& result);
    virtual bool shouldInterruptJavaScript();
    
    virtual bool locationbarVisible();
    virtual bool menubarVisible();
    virtual bool personalbarVisible();
    virtual bool statusbarVisible();
    virtual bool toolbarVisible();

    virtual void createEmptyDocument();
    virtual Range* markedTextRange() const;

    virtual String incomingReferrer() const;
    virtual String userAgent() const;

    virtual String mimeTypeForFileName(const String&) const;

    virtual void markMisspellingsInAdjacentWords(const VisiblePosition&);
    virtual void markMisspellings(const Selection&);

    virtual bool lastEventIsMouseUp() const;

    virtual bool passSubframeEventToSubframe(MouseEventWithHitTestResults &, Frame* subframePart = 0);
    virtual bool passWheelEventToChildWidget(Node*);
    
    virtual String overrideMediaType() const;

    virtual KJS::Bindings::Instance* getEmbedInstanceForWidget(Widget*);
    virtual KJS::Bindings::Instance* getObjectInstanceForWidget(Widget*);
    virtual KJS::Bindings::Instance* getAppletInstanceForWidget(Widget*);
    virtual KJS::Bindings::RootObject* bindingRootObject();
    
    virtual void registerCommandForUndo(PassRefPtr<EditCommand>);
    virtual void registerCommandForRedo(PassRefPtr<EditCommand>);
    virtual void clearUndoRedoOperations();
    virtual void issueUndoCommand();
    virtual void issueRedoCommand();
    virtual void issueCutCommand();
    virtual void issueCopyCommand();
    virtual void issuePasteCommand();
    virtual void issuePasteAndMatchStyleCommand();
    virtual void issueTransposeCommand();
    virtual void respondToChangedSelection(const Selection& oldSelection, bool closeTyping);
    void FrameAG::respondToChangedContents(const Selection& endingSelection);
    virtual bool shouldChangeSelection(const Selection& oldSelection, const Selection& newSelection, EAffinity affinity, bool stillSelecting) const;
    virtual void partClearedInBegin();

    virtual bool canGoBackOrForward(int distance) const;
    virtual void goBackOrForward(int distance);
    virtual int getHistoryLength();
    virtual KURL historyURL(int distance);
    virtual void handledOnloadEvents();

    virtual bool canPaste() const;
    virtual bool canRedo() const;
    virtual bool canUndo() const;
    virtual void print();

    bool keyPress(const PlatformKeyboardEvent&);
    virtual KURL originalRequestURL() const;
    virtual IntRect windowResizerRect() const;
    
protected:
    virtual bool isLoadTypeReload();

private:
    virtual bool passMouseDownEventToWidget(Widget*);
    FrameAGClient* m_client;
};

inline FrameAG* Ag(Frame* frame) { return static_cast<FrameAG*>(frame); }
inline const FrameAG* Ag(const Frame* frame) { return static_cast<const FrameAG*>(frame); }

} // namespace WebCore

#endif
