#include "config.h"
#include "ag.h"
#include "WebKitAGObjectData.h"
#include "DeprecatedCString.h"

namespace WebCore {

WebKitAGObjectData::WebKitAGObjectData() 
	: m_ag_params_size(0)
	, m_ag_paramNames(0)
	, m_ag_paramValues(0)
	, m_ag_serviceType(0)
	, m_ag_url(0)
	, m_ag_MultiMedia(false)
	, m_pCachedEmbedObject(NULL)
	, m_bIsInitialized(false)
{
	m_ag_objectData.data = 0;
	m_ag_objectData.size = 0;
}

WebKitAGObjectData::~WebKitAGObjectData()
{
	for(int i=0; i < m_ag_params_size; ++i)
	{
		delete [] (m_ag_paramNames[i]);
		delete [] (m_ag_paramValues[i]);
	}
	delete [] m_ag_paramNames;
	delete [] m_ag_paramValues;
	delete [] m_ag_serviceType;
	delete [] m_ag_url;
	delete [] m_ag_objectData.data;
#ifndef NDEBUG
	m_pCachedEmbedObject = NULL;
	m_bIsInitialized = false;
#endif
}

void WebKitAGObjectData::copyURL(const DeprecatedCString& url)
{
	assert(!m_bIsInitialized);
	int url_size = url.length() + 1;
	m_ag_url = new char[url_size];
	strncpy(m_ag_url, url.data(), url_size);
}

void WebKitAGObjectData::copyServiceType(const DeprecatedCString& serviceType)
{
	assert(!m_bIsInitialized);
	int serviceType_size = serviceType.length() + 1;
	m_ag_serviceType = new char[serviceType_size];
	strncpy(m_ag_serviceType, serviceType.data(), serviceType_size);
}

void WebKitAGObjectData::setIsMultimedia(bool isMultimedia)
{
	assert(!m_bIsInitialized);
	m_ag_MultiMedia = isMultimedia;
}

void WebKitAGObjectData::allocParams(int nParams)
{
	assert(!m_bIsInitialized);
	m_ag_params_size = nParams;
	m_ag_paramNames = (char**)new char*[m_ag_params_size];
	m_ag_paramValues = (char**)new char*[m_ag_params_size];
}

void WebKitAGObjectData::addParamValue(const DeprecatedCString& paramName, const DeprecatedCString& paramValue, const int& at)
{
	assert(!m_bIsInitialized);
	int paramName_size = paramName.length() + 1;
	int paramValue_size = paramValue.length() + 1;

	m_ag_paramNames[at] = new char[paramName_size];
	m_ag_paramValues[at] = new char[paramValue_size];
	strncpy(m_ag_paramNames[at], paramName.data(), paramName_size);
    strncpy(m_ag_paramValues[at], paramValue.data(), paramValue_size);
}

void WebKitAGObjectData::copyRawData(const char* rawData, const int size)
{
	assert(m_bIsInitialized);
	if(m_ag_objectData.data == NULL) {
		m_ag_objectData.size = size;
		m_ag_objectData.data = new char[m_ag_objectData.size];
		memcpy((void*)m_ag_objectData.data, rawData, m_ag_objectData.size);
	}
}

}
