#include "config.h"
#include "CachedEmbedObject.h"

#include "Cache.h"
#include "CachedResourceClient.h"
#include "CachedResourceClientWalker.h"
#include "loader.h"
#include <wtf/Vector.h>
#include "Request.h"

namespace WebCore {

CachedEmbedObject::CachedEmbedObject(DocLoader* dl, const String& url, CachePolicy cachePolicy)
    : CachedResource(url, EmbedObject, cachePolicy)
{
    setAccept("*/*");
    m_errorOccurred = false;
#if PLATFORM(AG)
	m_loading = true;
#endif
    // load the file
    cache()->loader()->load(dl, this, false);
#if !PLATFORM(AG)
	m_loading = true;
#endif
}

CachedEmbedObject::~CachedEmbedObject()
{
}

void CachedEmbedObject::ref(CachedResourceClient* c)
{
    CachedResource::ref(c);
    if (!m_loading)
        c->notifyFinished(this);
}

void CachedEmbedObject::data(PassRefPtr<SharedBuffer> data, bool allDataReceived)
{
    if (!allDataReceived)
        return;

	setEncodedSize(data.get() ? data.get()->size() : 0);
	m_data = data;
    m_loading = false;
    checkNotify();
}

void CachedEmbedObject::checkNotify()
{
    if (m_loading)
        return;

    CachedResourceClientWalker w(m_clients);
    while (CachedResourceClient* c = w.next())
        c->notifyFinished(this);
}

void CachedEmbedObject::error()
{
    m_loading = false;
    m_errorOccurred = true;
    checkNotify();
}

const char* CachedEmbedObject::getData(unsigned int* size)
{
	if(m_data != NULL) {
		if (size)
			*size = m_data.get()->size();
		return m_data.get()->data();
	} else {
		if(size)
			*size = 0;
		return NULL;
	}
}

}
