/*
* Copyright (C) 2003, 2006 Apple Computer, Inc.  All rights reserved.
*           (C) 2006 Graham Dennis (graham.dennis@gmail.com)
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef Settings_h
#define Settings_h

#include "AtomicString.h"
#include "KURL.h"

#if PLATFORM(AG)
#include "WebKitAGSettings.h"
#endif

namespace WebCore {

	class Page;

	enum EditableLinkBehavior {
		EditableLinkDefaultBehavior = 0,
		EditableLinkAlwaysLive,
		EditableLinkOnlyLiveWithShiftKey,
		EditableLinkLiveWhenNotFocused,
		EditableLinkNeverLive
	};

#if PLATFORM(AG)
	struct WebKitAGFontLanguageScriptMapEntry
	{
		int fontSize;
		AtomicString fontName;
		WebKitAGLanguageScript languageScript;
		AGEncodingType_t encodingType;
	};
#endif

	class Settings
#if PLATFORM(AG)
		: public WebKitAGSettings
#endif
	{
	public:
		Settings(Page*);

		void setStandardFontFamily(const AtomicString&);
		const AtomicString& standardFontFamily() const { return m_standardFontFamily; }

		void setFixedFontFamily(const AtomicString&);
		const AtomicString& fixedFontFamily() const { return m_fixedFontFamily; }

		void setSerifFontFamily(const AtomicString&);
		const AtomicString& serifFontFamily() const { return m_serifFontFamily; }

		void setSansSerifFontFamily(const AtomicString&);
		const AtomicString& sansSerifFontFamily() const { return m_sansSerifFontFamily; }

		void setCursiveFontFamily(const AtomicString&);
		const AtomicString& cursiveFontFamily() const { return m_cursiveFontFamily; }

		void setFantasyFontFamily(const AtomicString&);
		const AtomicString& fantasyFontFamily() const { return m_fantasyFontFamily; }

		void setMinimumFontSize(int);
		int minimumFontSize() const { return m_minimumFontSize; }

		void setMinimumLogicalFontSize(int);
		int minimumLogicalFontSize() const { return m_minimumLogicalFontSize; }

		void setDefaultFontSize(int);
		int defaultFontSize() const { return m_defaultFontSize; }

		void setDefaultFixedFontSize(int);
		int defaultFixedFontSize() const { return m_defaultFixedFontSize; }

		void setLoadsImagesAutomatically(bool);
		bool loadsImagesAutomatically() const { return m_loadsImagesAutomatically; }

		void setJavaScriptEnabled(bool);
		bool isJavaScriptEnabled() const { return m_isJavaScriptEnabled; }

		void setJavaScriptCanOpenWindowsAutomatically(bool);
		bool JavaScriptCanOpenWindowsAutomatically() const { return m_javaScriptCanOpenWindowsAutomatically; }

		void setJavaEnabled(bool);
		bool isJavaEnabled() const { return m_isJavaEnabled; }

		void setPluginsEnabled(bool);
		bool arePluginsEnabled() const { return m_arePluginsEnabled; }

		void setPrivateBrowsingEnabled(bool);
		bool privateBrowsingEnabled() const { return m_privateBrowsingEnabled; }

		void setDefaultTextEncodingName(const String&);
		const String& defaultTextEncodingName() const { return m_defaultTextEncodingName; }

		void setUserStyleSheetLocation(const KURL&);
		KURL userStyleSheetLocation() const { return m_userStyleSheetLocation; }

		void setShouldPrintBackgrounds(bool);
		bool shouldPrintBackgrounds() const { return m_shouldPrintBackgrounds; }

		void setTextAreasAreResizable(bool);
		bool textAreasAreResizable() const { return m_textAreasAreResizable; }

		void setEditableLinkBehavior(EditableLinkBehavior);
		EditableLinkBehavior editableLinkBehavior() const { return m_editableLinkBehavior; }

		void setUsesDashboardBackwardCompatibilityMode(bool);
		bool usesDashboardBackwardCompatibilityMode() const { return m_usesDashboardBackwardCompatibilityMode; }

		void setNeedsAdobeFrameReloadingQuirk(bool);
		bool needsAcrobatFrameReloadingQuirk() const { return m_needsAdobeFrameReloadingQuirk; }

		void setDOMPasteAllowed(bool);
		bool isDOMPasteAllowed() const { return m_isDOMPasteAllowed; }

		void setUsesPageCache(bool);
		bool usesPageCache() const { return m_usesPageCache; }

		void setShrinksStandaloneImagesToFit(bool);
		bool shrinksStandaloneImagesToFit() const { return m_shrinksStandaloneImagesToFit; }

		void setShowsURLsInToolTips(bool);
		bool showsURLsInToolTips() const { return m_showsURLsInToolTips; }

		void setFTPDirectoryTemplatePath(const String&);
		const String& ftpDirectoryTemplatePath() const { return m_ftpDirectoryTemplatePath; }

		void setForceFTPDirectoryListings(bool);
		bool forceFTPDirectoryListings() const { return m_forceFTPDirectoryListings; }

		void setDeveloperExtrasEnabled(bool);
		bool developerExtrasEnabled() const { return m_developerExtrasEnabled; }

	private:
		Page* m_page;
		String m_defaultTextEncodingName;
		String m_ftpDirectoryTemplatePath;
		KURL m_userStyleSheetLocation;
		AtomicString m_standardFontFamily;
		AtomicString m_fixedFontFamily;
		AtomicString m_serifFontFamily;
		AtomicString m_sansSerifFontFamily;
		AtomicString m_cursiveFontFamily;
		AtomicString m_fantasyFontFamily;
		EditableLinkBehavior m_editableLinkBehavior;
		int m_minimumFontSize;
		int m_minimumLogicalFontSize;
		int m_defaultFontSize;
		int m_defaultFixedFontSize;
		bool m_isJavaEnabled : 1;
		bool m_loadsImagesAutomatically : 1;
		bool m_privateBrowsingEnabled : 1;
		bool m_arePluginsEnabled : 1;
		bool m_isJavaScriptEnabled : 1;
		bool m_javaScriptCanOpenWindowsAutomatically : 1;
		bool m_shouldPrintBackgrounds : 1;
		bool m_textAreasAreResizable : 1;
		bool m_usesDashboardBackwardCompatibilityMode : 1;
		bool m_needsAdobeFrameReloadingQuirk : 1;
		bool m_isDOMPasteAllowed : 1;
		bool m_shrinksStandaloneImagesToFit : 1;
		bool m_usesPageCache: 1;
		bool m_showsURLsInToolTips : 1;
		bool m_forceFTPDirectoryListings : 1;
		bool m_developerExtrasEnabled : 1;

#if PLATFORM(AG)
	private:
		AtomicString m_stdEncoding;
		int m_pageWidth;
		int m_pageHeight;
		int m_topMargin;
		int m_bottomMargin;
		int m_leftMargin;
		int m_rightMargin;
		bool m_scaleToFit;
		bool m_autoLandscape;
		bool m_createHeader;
		int m_autoLandscapeThreshold;
		bool m_landscape;
		bool m_preWrap;
		bool m_convertImages;
		bool m_retainBG;
		bool m_forceColors;
		bool m_ulLinks;
		bool m_genStructure;
		unsigned int m_textColor;
		unsigned int m_bgColor;
		unsigned int m_linkColor;
		float m_fontSize;
		String m_defaultEncoding;
		bool m_forceDefaultEncoding;
		AGEmbedMultimedia_t m_mmEmbedFlag;

		WebKitAGLanguageScript m_defaultLanguageScript;
		WebKitAGFontLanguageScriptMapEntry m_fontLangScriptMap[AG_LANG_SCRIPT_COUNT];
		bool m_bIsIEGeneratedHtmlStream;
		
	public:
		~Settings(){}

		const AtomicString& stdEncoding() const { return m_stdEncoding; } 

		void setStdEncoding(const AtomicString& s) { m_stdEncoding = s; }



		void SetPageDimensions(int width, int height, int topMargin, int bottomMargin, int leftMargin, int rightMargin)
		{
			m_pageWidth = width;
			m_pageHeight = height;
			m_topMargin = topMargin;
			m_bottomMargin = bottomMargin;
			m_leftMargin = leftMargin;
			m_rightMargin = rightMargin;
		}

		void pageDimensions(int* width, int* height, int* topMargin, int* bottomMargin, int* leftMargin, int* rightMargin) const
		{
			*width = m_pageWidth;
			*height = m_pageHeight;
			*topMargin = m_topMargin;
			*bottomMargin = m_bottomMargin;
			*leftMargin = m_leftMargin;
			*rightMargin = m_rightMargin;
		}

		void SetScaleToFitPageDim(bool scaleToFit)
		{
			m_scaleToFit = scaleToFit;
		}

		void scaleToFitPageDim(bool* scaleToFit) const
		{
			*scaleToFit = m_scaleToFit;
		}

		void SetPreWrap(bool preWrap)
		{
			m_preWrap = preWrap;
		}

		void preWrap(bool* pre_wrap) const
		{
			*pre_wrap = m_preWrap;
		}

		void SetGenStructure(bool genStructure)
		{
			m_genStructure = genStructure;
		}

		void genStructure(bool* genStructure) const
		{
			*genStructure = m_genStructure;
		}

		void SetConvertImages(bool convert_images)
		{
			m_convertImages = convert_images;
		}

		void convertImages(bool* convert_images) const
		{
			*convert_images = m_convertImages;
		}

		void SetRetainBG(bool retain_BG)
		{
			m_retainBG = retain_BG;
		}

		void retainBG(bool* retain_BG) const
		{
			*retain_BG = m_retainBG;
		}

		void SetForceColors(bool forceColors)
		{
			m_forceColors = forceColors;
		}

		void forceColors(bool* forceColors) const
		{
			*forceColors = m_forceColors;
		}

		void SetULLinks(bool ul_links)
		{
			m_ulLinks = ul_links;
		}

		void ulLinks(bool* ul_links) const
		{
			*ul_links = m_ulLinks;
		}

		void SetBGColor(unsigned int color)
		{
			m_bgColor = color;
		}

		void bgColor(unsigned int *color) const
		{
			*color = m_bgColor;
		}

		void SetTextColor(unsigned int color)
		{
			m_textColor = color;
		}

		void textColor(unsigned int *color) const
		{
			*color = m_textColor;
		}

		void SetLinkColor(unsigned int color)
		{
			m_linkColor = color;
		}

		void linkColor(unsigned int *color) const
		{
			*color = m_linkColor;
		}

		void SetFontSize(float fontSize)
		{
			m_fontSize = fontSize;
		}

		void fontSize(float *fontSize) const
		{
			*fontSize = m_fontSize;
		}

		void SetDefaultEncoding(const char* encoding)
		{
			setDefaultTextEncodingName(encoding);
		}

		/*const String& defaultEncoding() const
		{
			return m_defaultEncoding;
		}*/

		void SetForceDefaultEncoding(bool force)
		{
			m_forceDefaultEncoding = force;
		}

		void forceDefaultEncoding(bool* force) const
		{
			*force = m_forceDefaultEncoding;
		}

		void SetJavaScript(bool java_script)
		{
			m_isJavaScriptEnabled = java_script;
		}

		void javaScript(bool* java_script) const
		{
			*java_script = m_isJavaScriptEnabled;
		}

		void SetMMEmbedFlag(AGEmbedMultimedia_t embedFlag)
		{
			m_mmEmbedFlag = embedFlag;
		}

		void mmEmbedFlag(AGEmbedMultimedia_t* embedFlag) const
		{
			*embedFlag = m_mmEmbedFlag;
		}

		void SetAutoLandscapeParams(bool autoLandscape, int autoLandscapeThreshold)
		{
			m_autoLandscape = autoLandscape;
			m_autoLandscapeThreshold = autoLandscapeThreshold;
		}

		void autoLandscapeParams(bool* autoLandscape, int* autoLandscapeThreshold) const
		{
			*autoLandscape = m_autoLandscape;
			*autoLandscapeThreshold = m_autoLandscapeThreshold;
		}

		void SetCreateHeader(bool createHeader)
		{
			m_createHeader = createHeader;
		}

		void createHeader(bool* createHeader) const
		{
			*createHeader = m_createHeader;
		}

		void SetLandscape(bool landscape)
		{
			m_landscape = landscape;
		}

		void landscape(bool* landscape) const
		{
			*landscape = m_landscape;
		}

		void SetDefaultLanguageScript(WebKitAGLanguageScript langScript)
		{
			m_defaultLanguageScript = langScript;
		}

		WebKitAGLanguageScript defaultLanguageScript() const
		{
			return m_defaultLanguageScript;
		}

		void SetFontLanguageScriptMapping(FontLanguageScriptMapEntry* fontLanguageScriptMap)
		{
			for (int i=0; i < AG_LANG_SCRIPT_COUNT; ++i)
			{
				m_fontLangScriptMap[i].languageScript = fontLanguageScriptMap[i].languageScript;
				m_fontLangScriptMap[i].fontSize = fontLanguageScriptMap[i].fontSize;
				if(fontLanguageScriptMap[i].fontName && fontLanguageScriptMap[i].fontNameLen)
					m_fontLangScriptMap[i].fontName = AtomicString(fontLanguageScriptMap[i].fontName, fontLanguageScriptMap[i].fontNameLen);
				else
					m_fontLangScriptMap[i].fontName = nullAtom;
				m_fontLangScriptMap[i].encodingType = fontLanguageScriptMap[i].encodingType;
			}
		}

		const WebKitAGFontLanguageScriptMapEntry* GetFontForLanguage(const WebKitAGLanguageScript language) const
		{
			return &m_fontLangScriptMap[language];
		}

		virtual void SetIsIEGeneratedHtmlStream(bool bIsIEGeneratedHtmlStream) {
			m_bIsIEGeneratedHtmlStream = bIsIEGeneratedHtmlStream;
		}

		virtual void isIEGeneratedHtmlStream(bool* pIsIEGeneratedHtmlStream) const {
			if(pIsIEGeneratedHtmlStream) *pIsIEGeneratedHtmlStream = m_bIsIEGeneratedHtmlStream;
		}

#endif

	};

} // namespace WebCore

#endif // Settings_h
