/*************************************************************************
*
* ADOBE CONFIDENTIAL
* ___________________
*
*  Copyright 2007 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/
//this include is necessary to build on mac.
//otherwise we get this compiler error: if you use new/delete make sure to include config.h at the top of the file
#include "config.h" 

#include "Font.h"
#include "FontData.h"
#include "FontFallbackList.h"
#include "GlyphBuffer.h"
#include "GraphicsContext.h"
#include "IntRect.h"
#include "ag.h"
#include "agUtils.h"

namespace WebCore {

	void Font::drawGlyphs(GraphicsContext* graphicsContext, const FontData* font, const GlyphBuffer& glyphBuffer, 
		int from, int numGlyphs, const FloatPoint& point) const
	{
		AGContext_t* context = graphicsContext->platformContext();
		// Set the text color to use for drawing.
		/*float red, green, blue, alpha;
		Color penColor = graphicsContext->strokeColor();
		penColor.getRGBA(red, green, blue, alpha);
		context->ag_backend->ag_set_color_rgba(context, red, green, blue, alpha);*/

		// Select our font face/size.
		const FontPlatformData& fontPlatformData = font->platformData();
		AGFontBackend_t* fontBE = fontPlatformData.fontServerBackend()->ag_font_backend;
		fontBE->ag_set_font_face(fontBE, font->m_font.fontFace());
		fontBE->ag_set_font_size(fontBE, font->m_font.size());

		// Select the scaled font.
		AGScaledFont_t* scaledFont = font->m_font.scaledFont();
		fontBE->ag_scaled_font_select_font(context, scaledFont);

		GlyphBufferGlyph* glyphs = const_cast<GlyphBufferGlyph*>(glyphBuffer.glyphs(from));

		float offset = point.x();
		for (int i = 0; i < numGlyphs; i++) {
			glyphs[i].x = offset;
			glyphs[i].y = point.y();
			offset += glyphBuffer.advanceAt(from + i);
		}
		AGFont_t* ag_font = font->m_font.font();
		AGEncoding_t* ag_font_encoding = font->m_font.encoding();
		if (! ag_font_encoding)
		{
			const char* cmap_identityH = "Identity-H";
			ag_font_encoding = fontBE->ag_create_encoding_from_cmap_name(fontBE, cmap_identityH, strlen(cmap_identityH));
		}

		AGMatrix_t sizeMatrix;
		ag_init_matrix_to_identity(&sizeMatrix);
		ag_matrix_init_scale(&sizeMatrix, font->m_font.size(), font->m_font.size());

		context->ag_backend->ag_show_glyphs(context, scaledFont, glyphs, numGlyphs, ag_font_encoding, &sizeMatrix, NULL);

		fontBE->ag_scaled_font_done_font(fontBE, scaledFont);
	}

	FloatRect Font::selectionRectForComplexText(const TextRun&, const TextStyle&, const IntPoint&, int h, int from, int to) const
	{
		agNotImplemented();
		return FloatRect();
	}

	void Font::drawComplexText(GraphicsContext*, const TextRun&, const TextStyle&, const FloatPoint&, int from, int to) const
	{
		agNotImplemented();
	}

	float Font::floatWidthForComplexText(const TextRun& run, const TextStyle& style) const
	{
		agNotImplemented();
		return 0;
	}

	int Font::offsetForPositionForComplexText(const TextRun& run, const TextStyle& style, int x, bool includePartialGlyphs) const
	{
		agNotImplemented();
		return 0;
	}

}
