/*************************************************************************
*
* ADOBE CONFIDENTIAL
* ___________________
*
*  Copyright 2007 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/
//this include is necessary to build on mac.
//otherwise we get this compiler error: if you use new/delete make sure to include config.h at the top of the file
#include "config.h" 

#include "Font.h"
#include "FontCache.h"
#include "FloatRect.h"
#include "FontData.h"
#include "FontDescription.h"
#include <wtf/MathExtras.h>
#include <unicode/uchar.h>
#include <unicode/unorm.h>
#include "ag.h"

namespace WebCore
{
	void FontData::platformInit()
	{  
		//m_isMLangFont = false;		
		AGFont_t* ag_font = m_font.font();
		const double metricsMultiplier = 1 * m_font.size();
		
		AGFontServerBackend* fontServerBE = m_font.fontServerBackend();
		AGFontBackend_t* fontBE = fontServerBE->ag_font_backend;

		AGFontData_t ag_font_data = fontBE->ag_get_font_data(fontBE, ag_font);

		m_ascent = lroundf(ag_font_data.ascent * metricsMultiplier);
		m_descent = lroundf(ag_font_data.descent * metricsMultiplier);
		m_ascent += floorf(((m_ascent + m_descent) * 0.15f) + 0.5f);
		m_xHeight = m_ascent * 0.56f;  // Best guess for xHeight for non-Truetype fonts.
		m_lineGap = lroundf(ag_font_data.lineGap * metricsMultiplier);
		m_lineSpacing = m_ascent + m_descent + m_lineGap;
	}

	void FontData::platformDestroy()
	{
		delete m_smallCapsFontData;
	}

	FontData* FontData::smallCapsFontData(const FontDescription& fontDescription) const
	{
		if (!m_smallCapsFontData) {
			FontPlatformData origFontPlatformData = platformData();
			m_smallCapsFontData = new FontData(FontPlatformData(origFontPlatformData.fontName().characters(), 
								origFontPlatformData.fontName().length(),
								origFontPlatformData.size() * 0.70f,
								origFontPlatformData.weight(),
								origFontPlatformData.italic(), 
								origFontPlatformData.encoding()));
		}
		return m_smallCapsFontData;
	}

	bool FontData::containsCharacters(const UChar* characters, int length) const
	{
		UChar* glyphBuffer = new UChar[length];
		AGFontServerBackend* fontServerBE = m_font.fontServerBackend();
		AGFontBackend_t* fontBE = fontServerBE->ag_font_backend;
		fontBE->ag_get_glyph_ids(fontBE, m_font.scaledFont(), (UChar*)characters, length, (UChar*)glyphBuffer);
		for (int i = 0; i < length; i++) {
			if (glyphBuffer[i] == 0xFFFF) {
				delete []glyphBuffer;
				return false;
			}
		}
		delete []glyphBuffer;
		return true;
	}

	void FontData::determinePitch()
	{
		AGFontServerBackend* fontServerBE = m_font.fontServerBackend();
		AGFontBackend_t* fontBE = fontServerBE->ag_font_backend;
		AGFontData_t ag_font_data = fontBE->ag_get_font_data(fontBE, m_font.font());
		m_treatAsFixedPitch = (ag_font_data.pitch == ag_fixedPitch);
	}

	float FontData::platformWidthForGlyph(Glyph glyph) const
	{
		AGFontServerBackend* fontServerBE = m_font.fontServerBackend();
		AGFontBackend_t* fontBE = fontServerBE->ag_font_backend;
		float width = fontBE->ag_get_glyph_width(fontBE, m_font.scaledFont(), (UChar)glyph);
		return width;
	}
}