//this include is necessary to build on mac.
//otherwise we get this compiler error: if you use new/delete make sure to include config.h at the top of the file
#include "config.h"

#include "FontPlatformDataAG.h"
#include "FontDescription.h"

#include "ag.h"
#include "agUtils.h"

namespace WebCore {

	AGFontServerBackend* AGFontServerBackend::theAGFontServerBackend = 0;
	AGFontBackend_t* AGFontServerBackend::ag_font_backend = 0;

	AGFontServerBackend::AGFontServerBackend(AGFontBackend_t* agFontBakend)
	{
		ag_font_backend = agFontBakend;
	}

	AGFontServerBackend::~AGFontServerBackend()
	{
	}
	
	AGFontServerBackend* AGFontServerBackend::CreateUniqueInstance(AGFontBackend_t* agFontBakend)
	{
		if(!theAGFontServerBackend)
		{
			theAGFontServerBackend = new AGFontServerBackend(agFontBakend);
		}
		return theAGFontServerBackend;
	}

	AGFontServerBackend* AGFontServerBackend::GetUniqueInstance()
	{
		return theAGFontServerBackend;
	}

	void AGFontServerBackend::DestroyUniqueInstance()
	{
		if(theAGFontServerBackend)
		{
			theAGFontServerBackend->ag_font_backend->ag_terminate_font_server(ag_font_backend);
			theAGFontServerBackend->ag_font_backend = 0;
		}
		theAGFontServerBackend = 0;
	}

	AGFontServerBackend* FontPlatformData::fontServerBackend() const
	{ 
		return m_fontServerBackend; 
	}
	
	AGScaledFont_t* FontPlatformData::scaledFont() const
	{ 
		return m_ag_scaled_font; 
	}

	FontPlatformData::~FontPlatformData()
	{
	}

	FontPlatformData::FontPlatformData(const UChar* fontName, int fontNameLen, int size, unsigned weight, bool italic, AGEncoding_t* encoding)
		: m_size(size)
		, m_fontName(fontName, fontNameLen)
		, m_ag_encoding(encoding)
		, m_ag_scaled_font(0)
		, m_bFontObjectsNeedDestruction(true)
	{
		m_weight = weight;
		m_size = size;
		m_italic = italic;
		AGMatrix_t m, ctm;
		m_fontServerBackend = AGFontServerBackend::GetUniqueInstance();
		
		unsigned ag_weight = (weight >= cBoldWeight) ? 700 : 400;
		m_ag_font = m_fontServerBackend->ag_font_backend->ag_create_font(m_fontServerBackend->ag_font_backend, (unsigned short*)fontName, (unsigned)fontNameLen, ag_weight, italic, 0);
		ag_init_matrix_to_identity(&m);
		ag_matrix_init_scale(&m, m_size, m_size);
		
		AGEncodingType_t encodingType = AG_ENCODING_PLATFORM;
#if PLATFORM(WIN)
		encodingType = AG_ENCODING_WIN_PLATFORM;
#elif PLATFORM(MAC)
		encodingType = AG_ENCODING_MAC_PLATFORM;
#endif
		if (! encoding)
			encoding = m_fontServerBackend->ag_font_backend->ag_create_encoding(m_fontServerBackend->ag_font_backend, encodingType);

		m_ag_scaled_font =  m_fontServerBackend->ag_font_backend->ag_scaled_font_create( m_fontServerBackend->ag_font_backend, m_ag_font, encoding, &m, &ctm);
	}

	void FontPlatformData::releaseObjects() {
		// Encodings are cache'd -- we don't care to release those
		if(m_bFontObjectsNeedDestruction && m_fontServerBackend && m_fontServerBackend->ag_font_backend) {
			if(m_ag_font) {
				m_fontServerBackend->ag_font_backend->ag_font_release(m_ag_font);
				m_ag_font = NULL;
			}
			if(m_ag_scaled_font) {
				m_fontServerBackend->ag_font_backend->ag_scaled_font_release(m_ag_scaled_font);
				m_ag_scaled_font = NULL;
			}
		}
	}
}
