#ifndef FontPlatformData_H
#define FontPlatformData_H

#include "config.h"
#include "PlatformString.h"
#include "StringImpl.h"
#include "ag.h"
#include "StringHash.h"
#include <wtf/HashMap.h>

namespace WebCore {

	class FontDescription;
	
	class AGFontServerBackend {
	public:
		static AGFontBackend_t* ag_font_backend;
		static AGFontServerBackend* CreateUniqueInstance(AGFontBackend_t* agFontBakend);
		static AGFontServerBackend* GetUniqueInstance();
		static void DestroyUniqueInstance();
	private:
		static AGFontServerBackend* theAGFontServerBackend;
		AGFontServerBackend(AGFontBackend_t* agFontBakend);
		AGFontServerBackend();
		~AGFontServerBackend();
	};


	class FontPlatformData
	{
	public:
		class Deleted {};

		// Used for deleted values in the font cache's hash tables.
		FontPlatformData(Deleted)
			: m_size(-1), m_ag_encoding(0), m_ag_font(0), m_fontServerBackend(AGFontServerBackend::GetUniqueInstance()), m_bFontObjectsNeedDestruction(true)
		{}

		FontPlatformData()
			: m_size(0), m_ag_encoding(0), m_ag_font(0), m_fontServerBackend(AGFontServerBackend::GetUniqueInstance()), m_bFontObjectsNeedDestruction(true)
		{}
		FontPlatformData(const UChar* fontName, int fontNameLen, int size, unsigned weight, bool italic, AGEncoding_t* encoding);
		~FontPlatformData();

		AGFontServerBackend* fontServerBackend() const;
		const String& fontName() const { return m_fontName; }
		char* fontFace() const { return 0; }
		AGScaledFont_t* scaledFont() const;
		AGFont_t* font() const { return m_ag_font; }
		AGEncoding_t* encoding() const {return m_ag_encoding; }

		int size() const { return m_size; }
		unsigned weight() const { return m_weight; }
		bool italic() const { return m_italic; }
		bool bold() const { return m_weight == 63; }
		void setFontObjectsNeedDestruction(bool v = false) { m_bFontObjectsNeedDestruction = v; }

		unsigned hash() const
		{ 
			unsigned hashCodes[4] = {
				CaseInsensitiveHash<String>::hash(m_fontName),
				m_size,
				static_cast<unsigned>(bold()) << 1 | static_cast<unsigned>(m_italic),
				m_ag_encoding ? m_ag_encoding->type : AG_ENCODING_DEFAULT
			};
			return StringImpl::computeHash(reinterpret_cast<UChar*>(hashCodes), 4 * sizeof(unsigned) / sizeof(UChar));
		}

		bool operator==(const FontPlatformData& other) const
		{
			AGEncodingType_t e1 = AG_ENCODING_DEFAULT, e2 = AG_ENCODING_DEFAULT;
			if (m_ag_encoding)
				e1 = m_ag_encoding->type;
			if (other.encoding())
				e2 = other.encoding()->type;
			return equalIgnoringCase(m_fontName, other.m_fontName) && m_size == other.m_size && bold() == other.bold() && m_italic == other.m_italic && e1 == e2;
		}

		void releaseObjects();

		FontPlatformData& operator=(const FontPlatformData& rhs) {
			m_size = rhs.m_size;
			m_weight = rhs.m_weight;
			m_italic = rhs.m_italic;
			m_fontName = rhs.m_fontName;
			m_ag_encoding = rhs.m_ag_encoding;
			m_ag_font = rhs.m_ag_font;
			m_ag_scaled_font = rhs.m_ag_scaled_font;
			m_fontServerBackend = rhs.m_fontServerBackend;
			m_bFontObjectsNeedDestruction = rhs.m_bFontObjectsNeedDestruction;
			return *this;
		}

	private:
		int m_size;
		unsigned m_weight;
		bool m_italic;
		String m_fontName;
		AGEncoding_t* m_ag_encoding;
		AGFont_t* m_ag_font;
		AGScaledFont_t* m_ag_scaled_font;
		AGFontServerBackend* m_fontServerBackend;
		bool m_bFontObjectsNeedDestruction;
		// Remember to update the copy constructor
	};
}
#endif //FontPlatformData_H
