//this include is necessary to build on mac.
//otherwise we get this compiler error: if you use new/delete make sure to include config.h at the top of the file
#include "config.h"

#if PLATFORM(AG)

#define _USE_MATH_DEFINES 1

#include "GraphicsContext.h"
#include "AffineTransform.h"
#include "KURL.h"
#include "Path.h"
#include <wtf/MathExtras.h>
#include "agUtils.h"
#include "ag.h"

namespace WebCore {

	class GraphicsContextPlatformPrivate {
	public:
		GraphicsContextPlatformPrivate();
		~GraphicsContextPlatformPrivate();

		AGContext_t* context;
	};

	GraphicsContextPlatformPrivate::GraphicsContextPlatformPrivate()
		:  context(0)
	{
	}

	GraphicsContextPlatformPrivate::~GraphicsContextPlatformPrivate()
	{
		//context->ag_backend->ag_destroy(context);
	}

#if PLATFORM(WIN)
	GraphicsContext::GraphicsContext(HDC)
	{
		agNotImplemented();
	}

	HDC GraphicsContext::getWindowsContext(bool /*supportAlphaBlend*/, const IntRect*)
	{
		agNotImplemented();
		return 0;
	}

	void GraphicsContext::releaseWindowsContext(HDC, bool /*supportAlphaBlend*/, const IntRect*)
	{
		agNotImplemented();
	}
#endif

	static inline void setColor(AGContext_t* cr, const Color& col)
	{
		float red, green, blue, alpha;
		col.getRGBA(red, green, blue, alpha);
		cr->ag_backend->ag_set_color_rgba(cr, red, green, blue, alpha);
	}

	static inline void fillRectSourceOver(AGContext_t* cr, const FloatRect& rect, const Color& col)
	{
		if(rect.width() == 0 || rect.height() == 0)
			return;

		setColor(cr, col);
		cr->ag_backend->ag_rect(cr, rect.x(), rect.y(), rect.width(), rect.height());
		cr->ag_backend->ag_fill(cr, AG_FILL_WINDING);
	}

	GraphicsContext::GraphicsContext(PlatformGraphicsContext* context)
		: m_common(createGraphicsContextPrivate())
		, m_data(new GraphicsContextPlatformPrivate)
	{
		m_data->context = context;
		setPaintingDisabled(!context);
		if (context) {
			// Make sure the context starts in sync with our state.
			setPlatformFillColor(fillColor());
			setPlatformStrokeColor(strokeColor());
		}
	}

	GraphicsContext::~GraphicsContext()
	{
		destroyGraphicsContextPrivate(m_common);
		delete m_data;
	}

	void GraphicsContext::setFocusRingClip(const IntRect& r)
	{
		agNotImplemented();
	}

	void GraphicsContext::clearFocusRingClip()
	{
		agNotImplemented();
	}

	AGContext_t* GraphicsContext::platformContext() const
	{
		ASSERT(!paintingDisabled());
		ASSERT(m_data->context);
		return m_data->context;
	}

	void GraphicsContext::savePlatformState()
	{
		m_data->context->ag_backend->ag_save(m_data->context);
	}

	void GraphicsContext::restorePlatformState()
	{
		m_data->context->ag_backend->ag_restore(m_data->context);
	}

	// Draws a filled rectangle with a stroked border.
	void GraphicsContext::drawRect(const IntRect& rect)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()) )
			return;
#endif

		AGContext_t* context = m_data->context;
		if (fillColor().alpha())
			fillRectSourceOver(context, rect, fillColor());

		if (strokeStyle() != NoStroke && strokeColor().alpha()) {
			// We do a fill of four rects to simulate the stroke of a border.
			Color oldFillColor = fillColor();
			if (oldFillColor != strokeColor())
				setColor(context, strokeColor());
			FloatRect rects[4] = {
				FloatRect(rect.x(), rect.y(), rect.width(), 1),
				FloatRect(rect.x(), rect.bottom() - 1, rect.width(), 1),
				FloatRect(rect.x(), rect.y() + 1, 1, rect.height() - 2),
				FloatRect(rect.right() - 1, rect.y() + 1, 1, rect.height() - 2)
			};
			fillRectSourceOver(context, rects[0], fillColor());
			fillRectSourceOver(context, rects[1], fillColor());
			fillRectSourceOver(context, rects[2], fillColor());
			fillRectSourceOver(context, rects[3], fillColor());
			if (oldFillColor != strokeColor())
				setColor(context, oldFillColor);
		}
	}

	// This is only used to draw borders.
	void GraphicsContext::drawLine(const IntPoint& point1, const IntPoint& point2)
	{
		if (paintingDisabled() || point1 == point2)
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(point1.y(), point2.y() - point1.y()))
			return;
#endif

		if (strokeStyle() == NoStroke || !strokeColor().alpha())
			return;

		float width = strokeThickness();

		FloatPoint p1 = point1;
		FloatPoint p2 = point2;
		bool isVerticalLine = (p1.x() == p2.x());


		// For odd widths, we add in 0.5 to the appropriate x/y so that the float arithmetic
		// works out.  For example, with a border width of 3, KHTML will pass us (y1+y2)/2, e.g.,
		// (50+53)/2 = 103/2 = 51 when we want 51.5.  It is always true that an even width gave
		// us a perfect position, but an odd width gave us a position that is off by exactly 0.5.
		if (strokeStyle() == DottedStroke || strokeStyle() == DashedStroke) {
			if (isVerticalLine) {
				p1.move(0, width);
				p2.move(0, -width);
			} else {
				p1.move(width, 0);
				p2.move(-width, 0);
			}
		}

		if (((int)width) % 2) {
			if (isVerticalLine) {
				// We're a vertical line.  Adjust our x.
				p1.move(0.5f, 0.0f);
				p2.move(0.5f, 0.0f);
			} else {
				// We're a horizontal line. Adjust our y.
				p1.move(0.0f, 0.5f);
				p2.move(0.0f, 0.5f);
			}
		}

		int patWidth = 0;
		switch (strokeStyle()) {
	case NoStroke:
	case SolidStroke:
		break;
	case DottedStroke:
		patWidth = (int)width;
		break;
	case DashedStroke:
		patWidth = 3 * (int)width;
		break;
		}

		AGContext_t* context = m_data->context;
		m_data->context->ag_backend->ag_save(context);

		//m_data->context->ag_backend->ag_setAntialias(context, false);

		if (patWidth) {
			// Do a rect fill of our endpoints.  This ensures we always have the
			// appearance of being a border.  We then draw the actual dotted/dashed line.
			setColor(context, strokeColor());  // The save/restore make it safe to mutate the fill color here without setting it back to the old color.
			if (isVerticalLine) {
				fillRectSourceOver(context, FloatRect(p1.x() - width / 2, p1.y() - width, width, width), strokeColor());
				fillRectSourceOver(context, FloatRect(p2.x() - width / 2, p2.y(), width, width), strokeColor());
			} else {
				fillRectSourceOver(context, FloatRect(p1.x() - width, p1.y() - width / 2, width, width), strokeColor());
				fillRectSourceOver(context, FloatRect(p2.x(), p2.y() - width / 2, width, width), strokeColor());
			}

			// Example: 80 pixels with a width of 30 pixels.
			// Remainder is 20.  The maximum pixels of line we could paint
			// will be 50 pixels.
			int distance = (isVerticalLine ? (point2.y() - point1.y()) : (point2.x() - point1.x())) - 2*(int)width;
			int remainder = distance % patWidth;
			int coverage = distance - remainder;
			int numSegments = coverage / patWidth;

			float patternOffset = 0.0f;
			// Special case 1px dotted borders for speed.
			if (patWidth == 1)
				patternOffset = 1.0f;
			else {
				bool evenNumberOfSegments = numSegments % 2 == 0;
				if (remainder)
					evenNumberOfSegments = !evenNumberOfSegments;
				if (evenNumberOfSegments) {
					if (remainder) {
						patternOffset += patWidth - remainder;
						patternOffset += remainder / 2;
					} else
						patternOffset = patWidth / 2;
				} else {
					if (remainder)
						patternOffset = (patWidth - remainder)/2;
				}
			}


			float dottedLine[2] = { patWidth, patWidth };
			m_data->context->ag_backend->ag_set_line_dash(context, patternOffset, 2, dottedLine);
		}

		m_data->context->ag_backend->ag_moveto(context, p1.x(), p1.y());
		m_data->context->ag_backend->ag_lineto(context, p2.x(), p2.y());
		m_data->context->ag_backend->ag_stroke(context);
		m_data->context->ag_backend->ag_restore(context);
	}


	// This method is only used to draw the little circles used in lists.
	void GraphicsContext::drawEllipse(const IntRect& rect)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		int x = rect.x();
		int y = rect.y();
		float w = (float)rect.width();
		float h = (float)rect.height();
		float hRadius = w / 2;
		float vRadius = h / 2;

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);

		context->ag_backend->ag_begin_path(context);
		context->ag_backend->ag_arc(context, x + hRadius, y + vRadius, hRadius, vRadius, 0, 360, true);
		context->ag_backend->ag_close_path(context);

		if (fillColor().alpha()) {
			//check fill color , do we need to set it here
			//setColor(context, fillColor());
			context->ag_backend->ag_fill(context, AG_FILL_WINDING);
		}

		if (strokeStyle() != NoStroke) {
			//check line width  and stroke color here, do we need to set them here
			/*setColor(context, strokeColor());
			unsigned penWidth = pen().width();
			if (penWidth == 0) 
			penWidth++;
			context->ag_backend->ag_set_line_width(context, pen().width());*/
			context->ag_backend->ag_stroke(context);
		}
		context->ag_backend->ag_restore(context);
	}

	void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSpan)
	{
		if (paintingDisabled() || strokeStyle() == NoStroke || strokeThickness() <= 0.0f || !strokeColor().alpha())
			return;

		int x = rect.x();
		int y = rect.y();
		float w = (float)rect.width();
		float h = (float)rect.height();
		float hRadius = w / 2;
		float vRadius = h / 2;

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);
		context->ag_backend->ag_arc(context, x + hRadius, y + vRadius, hRadius, vRadius, startAngle, startAngle + angleSpan, true);
		setColor(context, strokeColor());
		context->ag_backend->ag_set_line_width(context, strokeThickness());
		context->ag_backend->ag_stroke(context);
		context->ag_backend->ag_restore(context);
	}

	void GraphicsContext::drawConvexPolygon(size_t npoints, const FloatPoint* points, bool shouldAntialias)
	{
		if (paintingDisabled() || !fillColor().alpha() && (strokeThickness() <= 0 || strokeStyle() == NoStroke))
			return;

		if (npoints <= 1)
			return;

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);

		//context->ag_backend_ag_set _antialias(context, shouldAntialias);
		//context->ag_backend->ag_begin_path
		context->ag_backend->ag_moveto(context, points[0].x(), points[0].y());
		for (size_t i = 1; i < npoints; i++)
			context->ag_backend->ag_lineto(context, points[i].x(), points[i].y());
		context->ag_backend->ag_close_path(context);

		if (fillColor().alpha()) {
			//check fill color , do we need to set it here
			//setColor(context, fillColor());
			context->ag_backend->ag_fill(context, AG_FILL_EVEN_ODD);
		}

		if (strokeStyle() != NoStroke) {
			//check stroke color and line width, do we need to set them here
			/*setColor(context, strokeColor());			
			context->ag_backend->ag_set_line_width(context, pen().width());*/
			context->ag_backend->ag_stroke(context);
		}
		context->ag_backend->ag_restore(context);
	}

	void GraphicsContext::fillRect(const IntRect& rect, const Color& color)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		if (color.alpha())
			fillRectSourceOver(m_data->context, rect, color);
	}

	void GraphicsContext::fillRect(const FloatRect& rect, const Color& color)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		if (color.alpha())
			fillRectSourceOver(m_data->context, rect, color);
	}

	void GraphicsContext::fillRoundedRect(const IntRect& rect, const IntSize& topLeft, const IntSize& topRight, const IntSize& bottomLeft, const IntSize& bottomRight, const Color& color)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::addRoundedRectClip(const IntRect& rect, const IntSize& topLeft, const IntSize& topRight,
    const IntSize& bottomLeft, const IntSize& bottomRight)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		float width = rect.width();
		float height = rect.height();
		
		if (width <= 0 || height <= 0)
			return;

		if (width < topLeft.width() + topRight.width()
			|| width < bottomLeft.width() + bottomRight.width()
			|| height < topLeft.height() + bottomLeft.height()
			|| height < topRight.height() + bottomRight.height())
		{
			clip(rect);
			return;
		}

		AGContext_t* context = m_data->context;
		float x = rect.x();
		float y = rect.y();
		
		context->ag_backend->ag_roundedRect_clip(context, x, y , width, height, topLeft.width(), topRight.width(), bottomLeft.width(), bottomRight.width());
	}

	void GraphicsContext::clip(const IntRect& rect)
	{
		if (paintingDisabled())
			return;
		
#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		m_data->context->ag_backend->ag_clip(m_data->context, rect.x(), rect.y(), rect.width(), rect.height());
	}

	void GraphicsContext::clipOut(const IntRect& rect)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::clipOutEllipseInRect(const IntRect& rect)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(rect.y(), rect.height()))
			return;
#endif

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_inner_roundedRect_clip(context, rect.x() , rect.y() , rect.width(), rect.height(), thickness);
	}

	void GraphicsContext::setShadow(const IntSize& size, int blur, const Color& color)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::clearShadow()
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::setMiterLimit(float limit)
	{
		if (paintingDisabled())
			return;

		platformContext()->ag_backend->ag_set_miter_limit(platformContext(), limit);
	}

	void GraphicsContext::setAlpha(float alpha)
	{
		if (paintingDisabled())
			return;

		platformContext()->ag_backend->ag_set_alpha(platformContext(), alpha);
	}

	void GraphicsContext::clearRect(const FloatRect& r)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::strokeRect(const FloatRect& r, float lineWidth)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(r.y(), r.height()))
			return;
#endif

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);

		context->ag_backend->ag_save(context);
		setPlatformStrokeThickness(lineWidth);
		context->ag_backend->ag_moveto(context, r.x(), r.y());
		context->ag_backend->ag_lineto(context, r.x(), r.y() - r.height());
		context->ag_backend->ag_lineto(context, r.x() + r.width(), r.y() - r.height());
		context->ag_backend->ag_lineto(context, r.x() + r.width(), r.y());
		context->ag_backend->ag_lineto(context, r.x() , r.y());
		context->ag_backend->ag_stroke(context);
		context->ag_backend->ag_restore(context);
	}

	void GraphicsContext::setLineCap(LineCap cap)
	{
		if (paintingDisabled())
			return;

		switch (cap) {
		case ButtCap:
			platformContext()->ag_backend->ag_set_line_cap(platformContext(), AG_BUTT_CAP);
			break;
		case RoundCap:
			platformContext()->ag_backend->ag_set_line_cap(platformContext(), AG_ROUND_CAP);
			break;
		case SquareCap:
			platformContext()->ag_backend->ag_set_line_cap(platformContext(), AG_SQUARE_CAP);
			break;
		}
	}

	void GraphicsContext::setLineJoin(LineJoin join)
	{
		if (paintingDisabled())
			return;

		switch (join) {
		case MiterJoin:
			platformContext()->ag_backend->ag_set_line_join(platformContext(), AG_MITER_JOIN);
			break;
		case RoundJoin:
			platformContext()->ag_backend->ag_set_line_join(platformContext(), AG_ROUND_JOIN);
			break;
		case BevelJoin:
			platformContext()->ag_backend->ag_set_line_join(platformContext(), AG_BEVEL_JOIN);
			break;
		}
	}

	void GraphicsContext::beginPath()
	{
		agNotImplemented();
	}

	void GraphicsContext::addPath(const Path& path)
	{
		agNotImplemented();
	}

	void GraphicsContext::clip(const Path& path)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::clipOut(const Path& path)
	{
		//ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::scale(const FloatSize& size)
	{
		if (paintingDisabled())
			return;
		ag_scale(platformContext(), size.width(), size.height());
	}

	void GraphicsContext::rotate(float angle)
	{
		if (paintingDisabled())
			return;
		ag_rotate(platformContext(), angle);
	}

	void GraphicsContext::translate(float x, float y)
	{
		AGContext_t* context = m_data->context;
		ag_translate(context, x, y);
	}

	void GraphicsContext::concatCTM(const AffineTransform& transform)
	{
		agNotImplemented();
	}
	FloatRect GraphicsContext::roundToDevicePixels(const FloatRect& frect)
	{
		FloatRect result;
		double x = frect.x();
		double y = frect.y();
		AGContext_t* context = m_data->context;
		ag_user_to_device(context,&x,&y);
		x = round(x);
		y = round(y);
		ag_device_to_user(context,&x,&y);
		result.setX((float)x);
		result.setY((float)y);
		x = frect.width();
		y = frect.height();
		ag_user_to_device_distance(context,&x,&y);
		x = round(x);
		y = round(y);
		ag_device_to_user_distance(context,&x,&y);
		result.setWidth((float)x);
		result.setHeight((float)y);
		return result; 
	}

	void GraphicsContext::drawLineForText(const IntPoint& point, int width, bool printing)
	{
		if (paintingDisabled())
			return;

#if HANDLE_PAGE_BREAKS
		if (isDrawnOnLastPage(point.y(), 0))
			return;
#endif

		if (width <= 0)
			return;

		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);

		float x = point.x();
		float y = point.y();
		float lineLength = width;

		// Use a minimum thickness of 0.5 in user space.
		// See http://bugs.webkit.org/show_bug.cgi?id=4255 for details of why 0.5 is the right minimum thickness to use.
		float thickness = 0.5f > strokeThickness() ? 0.5f : strokeThickness();

		if (!printing) {
			// On screen, use a minimum thickness of 1.0 in user space (later rounded to an integral number in device space).
			float adjustedThickness = 1.0f > strokeThickness() ? 1.0f : strokeThickness();

			// FIXME: This should be done a better way.
			// We try to round all parameters to integer boundaries in device space. If rounding pixels in device space
			// makes our thickness more than double, then there must be a shrinking-scale factor and rounding to pixels
			// in device space will make the underlines too thick.
			FloatRect lineRect = FloatRect(x, y, lineLength, adjustedThickness);
			if (lineRect.size().height() < thickness * 2.0) {
				x = lineRect.x();
				y = lineRect.y();
				lineLength = lineRect.size().width();
				thickness = lineRect.size().height();
				//platformContext()->ag_backend->ag_setAntialias(platformContext(), false);
			}
		}

		if (fillColor() != strokeColor())
			setColor(platformContext(), strokeColor());

		setPlatformStrokeThickness(thickness);

		//fillRectSourceOver(platformContext(), FloatRect(x, y, lineLength, thickness), fillColor());
		context->ag_backend->ag_moveto(context, point.x(), point.y());
		context->ag_backend->ag_lineto(context, point.x() + lineLength, point.y());
		context->ag_backend->ag_stroke(context);
		context->ag_backend->ag_restore(context);
	}


	void GraphicsContext::setURLForRect(const KURL& link, const IntRect& destRect)
	{
		//WEBC: TODO:implement this
		agNotImplemented();
	}

	void GraphicsContext::setPlatformTextDrawingMode(int mode)
	{
		//WEBC: TODO:implement this
		agNotImplemented();
	}

	void GraphicsContext::setPlatformStrokeColor(const Color& color)
	{
		if (paintingDisabled())
			return;
		setColor(platformContext(), color);
	}

	void GraphicsContext::setPlatformStrokeThickness(float thickness)
	{
		if (paintingDisabled())
			return;
		platformContext()->ag_backend->ag_set_line_width(platformContext(), thickness);
	}

	void GraphicsContext::setPlatformFillColor(const Color& color)
	{
		if (paintingDisabled())
			return;
		setColor(platformContext(), color);
	}

#if HANDLE_PAGE_BREAKS
	static bool chopImage(int top, int height, int width, int currBottomLine, int prevBottomLine, int pageHeight)
	{
		int bottom = top + height;

		if( (bottom > currBottomLine || (top <= prevBottomLine && bottom > prevBottomLine))
			&& width/(height*1.0) < 0.27
			&& height < pageHeight 
		  )
		{
			float blankRatio = 0.0f;
			
			if(bottom > currBottomLine)
				blankRatio = (currBottomLine - top + 1)/(pageHeight*1.0);
			else if(top <= prevBottomLine && bottom > prevBottomLine)
				blankRatio = (prevBottomLine - top + 1)/(pageHeight*1.0);
			

			if(blankRatio > 0.1f)
				return true;
		}

		return false;
	}

	static bool handleMultimediaBreak(AGContext_t* context, int* topM, int* heightM)
	{
		if(!context)
		{
			ASSERT(false);
			return false;
		}

		int prevBottomLine = context->ag_backend->ag_get_prev_bottom_line(context);
		int frameOffset = context->ag_backend->ag_get_frame_offset(context);
		int currBottomLine = context->ag_backend->ag_get_curr_bottom_line(context);
		int pageHeight = context->ag_backend->ag_get_scaled_content_height(context);

		int top = *topM + frameOffset;
		int bottom = top + *heightM;
		int height = *heightM;

		if(bottom <= prevBottomLine)
		{
			return false;
		}
		else
		{
			if(top > prevBottomLine && bottom > currBottomLine)
			{
				int upper = currBottomLine - top + 1;
				int lower = height - upper;

				if(upper > lower)
				{
					height = upper;
				}
				else
				{
					return false;
				}
			}
			else if(top <= prevBottomLine)
			{
				int upper = prevBottomLine - top + 1;
				int lower = height - upper;

				if(upper > lower)
				{
					return false;
				}
				else
				{
					int pageTop = currBottomLine - pageHeight + 1;

					if(bottom <= currBottomLine)
					{
						top = pageTop > top ? pageTop : top;
						height = bottom - top;
					}
					else //bottom > currBottomLine
					{
						int lowerNext = bottom - currBottomLine;
						int upperNext = height - lowerNext;

						if(upperNext > lowerNext)
						{
							top = pageTop > top ? pageTop : top;
							height = currBottomLine - top;
						}
						else
						{
							return false;
						}
					}
				}		
			}
		}

		*topM = top - frameOffset;
		*heightM = height;
		
		return true;
	}

	bool GraphicsContext::isDrawnOnLastPage(int top, int height)
	{
		AGContext_t* agContext = platformContext();

		if(!agContext)
		{
			ASSERT(false);
			return false;
		}

		int prevBottomLine = agContext->ag_backend->ag_get_prev_bottom_line(agContext);
		int frameOffset = agContext->ag_backend->ag_get_frame_offset(agContext);
		
		top = top + frameOffset;
		int bottom = top + height;

		bottom = bottom >= top ? bottom : top;
		
		if(bottom <= prevBottomLine)
		{
			return true;
		}

		return false;
	}

	bool GraphicsContext::isContentDrawableOnCurrPage(int top, int width, int height, bool isImage)
	{
		AGContext_t* agContext = platformContext();

		if(!agContext)
		{
			ASSERT(false);
			return true;
		}

		int prevBottomLine = agContext->ag_backend->ag_get_prev_bottom_line(agContext);
		int currBottomLine = agContext->ag_backend->ag_get_curr_bottom_line(agContext);
		int pageHeight = agContext->ag_backend->ag_get_scaled_content_height(agContext);
		int frameOffset = agContext->ag_backend->ag_get_frame_offset(agContext);
		
		top = top + frameOffset;
		int bottom = top + height;		

		if( isImage && chopImage(top, height, width, currBottomLine, prevBottomLine, pageHeight) )
		{
			return true;
		}	

		if(bottom > currBottomLine && height < pageHeight)
		{
			agContext->ag_backend->ag_set_overlap_height_next_page(agContext, currBottomLine - top + 1);
			return false;
		}
		else if(bottom <= prevBottomLine && height < pageHeight)
		{
			return false;
		}

		return true;
	}
#endif

	//----------------------------------------------------------------------------------------------------------------------
	//Description:
	//	This is a custom command implemented in assisting creation of tags, link annotations, named destinations and
	//	any other functionality associated with html that cannot be displayed but requires to be captured at render time.
	//@params:
	//	data:	data contains all the relevant information regarding the html tag being currently rendered.
	//NOTE:
	//	The x,y location of the tag has been transformed using ctm in AGContext to get it right on the rendered surface.
	//----------------------------------------------------------------------------------------------------------------------
	void GraphicsContext::setHTMLTagInfo(void* data)
	{
		AGContext_t* context = m_data->context;
		AGHTMLTagData_t* tag_data = (AGHTMLTagData_t*)data;

#if HANDLE_PAGE_BREAKS
		if(tag_data->isMultiMedia)
		{
			int top = tag_data->y;
			int height = tag_data->height;
			bool shouldDraw = handleMultimediaBreak(context, &top, &height);

			if(shouldDraw)
			{
				tag_data->y = top;
				tag_data->height = height;			
			}
			else
			{
				tag_data->isMultiMedia = false;
			}
		}
		
		else if(tag_data->isFormField && !isContentDrawableOnCurrPage(tag_data->y, tag_data->width, tag_data->height, false))
		{
			tag_data->isFormField = false;
		}

		if(tag_data->isLink && isDrawnOnLastPage(tag_data->y, tag_data->height))
		{
			tag_data->isLink = false;
		}
#endif		

		if(tag_data->tagState == AG_START_TAG)
			context->ag_backend->ag_start_current_tag_info(context, tag_data);
		else
			context->ag_backend->ag_stop_current_tag_info(context, tag_data);
	}


	void GraphicsContext::drawLineForMisspellingOrBadGrammar(const IntPoint& point, int width, bool grammar)
    {
        //ASSERT(false);
		agNotImplemented();
	}

	void GraphicsContext::setCompositeOperation(CompositeOperator mode) 
	{ 
		//ASSERT(false);
		agNotImplemented();
#if 0
		if (paintingDisabled())
			return;

		CGBlendMode target = kCGBlendModeNormal;   
		switch (mode) {
		case CompositeClear:
			target = kCGBlendModeClear;
			break;
		case CompositeCopy:
			target = kCGBlendModeCopy;
			break;
		case CompositeSourceOver:
			//kCGBlendModeNormal
			break;
		case CompositeSourceIn:
			target = kCGBlendModeSourceIn;
			break;
		case CompositeSourceOut:
			target = kCGBlendModeSourceOut;
			break;
		case CompositeSourceAtop:
			target = kCGBlendModeSourceAtop;
			break;
		case CompositeDestinationOver:
			target = kCGBlendModeDestinationOver;
			break;
		case CompositeDestinationIn:
			target = kCGBlendModeDestinationIn;
			break;
		case CompositeDestinationOut:
			target = kCGBlendModeDestinationOut;
			break;
		case CompositeDestinationAtop:
			target = kCGBlendModeDestinationAtop;
			break;
		case CompositeXOR:
			target = kCGBlendModeXOR;
			break;
		case CompositePlusDarker:
			target = kCGBlendModePlusDarker;
			break;
		case CompositeHighlight:
			// currently unsupported
			break;
		case CompositePlusLighter:
			target = kCGBlendModePlusLighter;
			break;
		}
		CGContextSetBlendMode(platformContext(), target);
#endif
	}

	void GraphicsContext::drawFocusRing(const Color& color)
	{
		//ASSERT(false);
		agNotImplemented();
#if 0
		if (paintingDisabled())
			return;

		float radius = (focusRingWidth() - 1) / 2.0f;
		int offset = radius + focusRingOffset();
		CGColorRef colorRef = color.isValid() ? cgColor(color) : 0;

		CGMutablePathRef focusRingPath = CGPathCreateMutable();
		const Vector<IntRect>& rects = focusRingRects();
		unsigned rectCount = rects.size();
		for (unsigned i = 0; i < rectCount; i++)
			CGPathAddRect(focusRingPath, 0, CGRectInset(rects[i], -offset, -offset));

		CGContextRef context = platformContext();
		CGContextSaveGState(context);

		CGContextBeginPath(context);
		CGContextAddPath(context, focusRingPath);

		wkDrawFocusRing(context, colorRef, radius);

		CGColorRelease(colorRef);

		CGPathRelease(focusRingPath);

		CGContextRestoreGState(context);
#endif
	}

	void GraphicsContext::beginTransparencyLayer(float opacity)
	{
		AGContext_t* context = m_data->context;
		context->ag_backend->ag_save(context);
		context->ag_backend->ag_set_transparency(context, opacity);
	}

	void GraphicsContext::endTransparencyLayer()
	{
		AGContext_t* context = m_data->context;
		//context->ag_backend->ag_end_transparency(context);
		context->ag_backend->ag_restore(context);
	}

} // namespace WebCore

#endif //PLATFORM(AG)
