#ifndef __AG_H_
#define __AG_H_

#include <WebKitAGResourceHandleClient.h>
#include <WebKitAGLanguage.h>
//WEBC: TODO: Define the correct values for ACCB1,ACCB2. Instead of this, use the WEBKITAG_PROTO* macros from WebKitAGConfig.h
#ifndef ACCB1
#if defined(WIN32)
#define ACCB1  
#define ACCB2  __cdecl
#else
//#error Handle this for mac
#define ACCB1  
#define ACCB2  
#endif
#endif


//WEBC: TODO: find a better way to define UChar using "platform/UChar.h"
#if defined(WIN32)
	typedef wchar_t UChar;
#else
	typedef uint16_t UChar;
#endif

#if defined(WIN32)
#include <stdio.h>
#define agNotImplemented() do { \
	char buf[256] = {0}; \
	_snprintf(buf, sizeof(buf), "AG::UNIMPLEMENTED: %s:%d\n", __FILE__, __LINE__); \
	fprintf(stderr, "%s", buf); \
} while (0)
#else
#define agNotImplemented()
#endif

#define HANDLE_PAGE_BREAKS 1

typedef enum _AGStatus
{
	AG_STATUS_SUCCESS,
	AG_STATUS_PAGE_SIZE_IS_INVALID,
	AG_STATUS_INVALID_CONTEXT,
	AG_STATUS_FAIL_SURFACE_ALREADY_FINISHED,
	AG_STATUS_FAILED_INIT_FONT_SERVER,
	AG_STATUS_HTML_TAG_MISMATCH,
	AG_STATUS_FOUND_UNSUPPORTED_CHAR,
	AG_STATUS_FAILURE
} AGStatus_t;

typedef enum _AGEncodingType
{
	AG_ENCODING_DEFAULT = 0,
	AG_ENCODING_WIN_PREDEFINED_BEGIN = AG_ENCODING_DEFAULT,
	AG_ENCODING_MAC_PLATFORM = 1,
	AG_ENCODING_WIN_PLATFORM = 2,
	AG_ENCODING_PLATFORM = 3,
	AG_ENCODING_UNICODE_BE = 8,
	AG_ENCODING_UNICODE_LE = 9,
	AG_ENCODING_UNICODE_3_BE = 12,
	AG_ENCODING_UNICODE_3_LE = 13,
	AG_ENCODING_WIN_PREDEFINED_END = AG_ENCODING_UNICODE_3_LE,

	AG_ENCODING_CODEPAGE_WIN_JAPANESE = 932,
	AG_ENCODING_WIN_CODEPAGES_BEGIN = AG_ENCODING_CODEPAGE_WIN_JAPANESE,
	AG_ENCODING_CODEPAGE_WIN_CHS = 936,
	AG_ENCODING_CODEPAGE_WIN_KOREAN = 949,
	AG_ENCODING_CODEPAGE_WIN_CHT = 950,
	AG_ENCODING_CODEPAGE_WIN_CE = 1250,
	AG_ENCODING_CODEPAGE_WIN_CYRILLIC = 1251,
	AG_ENCODING_CODEPAGE_WIN_ROMAN = 1252,
	AG_ENCODING_CODEPAGE_WIN_GREEK = 1253,
	AG_ENCODING_CODEPAGE_WIN_TURKISH = 1254,
	AG_ENCODING_CODEPAGE_WIN_HEBREW = 1255,
	AG_ENCODING_CODEPAGE_WIN_ARABIC = 1256,
	AG_ENCODING_CODEPAGE_WIN_BALTIC = 1257,
	AG_ENCODING_WIN_CODEPAGES_END = AG_ENCODING_CODEPAGE_WIN_BALTIC,

	AG_ENCODING_CODEPAGE_MAC_ROMAN = -10000,
	AG_ENCODING_MAC_CODEPAGES_BEGIN = AG_ENCODING_CODEPAGE_MAC_ROMAN,
	AG_ENCODING_CODEPAGE_MAC_JAPANESE,
	AG_ENCODING_CODEPAGE_MAC_CHS,
	AG_ENCODING_CODEPAGE_MAC_KOREAN,
	AG_ENCODING_CODEPAGE_MAC_CHT,
	AG_ENCODING_CODEPAGE_MAC_ICELANDIC,
	AG_ENCODING_CODEPAGE_MAC_CE,
	AG_ENCODING_CODEPAGE_MAC_CROATIAN,
	AG_ENCODING_CODEPAGE_MAC_ROMANIAN,
	AG_ENCODING_CODEPAGE_MAC_CYRILLIC,
	AG_ENCODING_CODEPAGE_MAC_UKRAINIAN,
	AG_ENCODING_CODEPAGE_MAC_GREEK,
	AG_ENCODING_CODEPAGE_MAC_TURKISH,
	AG_ENCODING_CODEPAGE_MAC_HEBREW,
	AG_ENCODING_CODEPAGE_MAC_ARABIC,
	AG_ENCODING_CODEPAGE_MAC_VIETNAMESE,
	AG_ENCODING_MAC_CODEPAGES_END = AG_ENCODING_CODEPAGE_MAC_VIETNAMESE,

	AG_ENCODING_CODEPAGE_GB18030,
	AG_ENCODING_CODEPAGE_PI_BEGIN = AG_ENCODING_CODEPAGE_GB18030,
	AG_ENCODING_CODEPAGE_HKSCS = 100000,
	AG_ENCODING_CODEPAGE_PI_END = AG_ENCODING_CODEPAGE_HKSCS,

	AG_ENCODING_CMAP_IDENTITY_H = AG_ENCODING_CODEPAGE_PI_END+1
} AGEncodingType_t;

typedef struct _AGFont
{
	void* data;
    unsigned int refCount;
} AGFont_t;

typedef struct AGEncoding_t
{
	AGEncodingType_t type;
	void* data;
    unsigned int refCount;
} AGEncoding_t;

typedef enum _AGPitch
{
	ag_fixedPitch,
	ag_variablePitch
} AGPitch_t;

typedef enum _AGFillRule
{
	AG_FILL_WINDING,
	AG_FILL_EVEN_ODD
} AGFillRule_t;

typedef enum _AGPlayer
{
	AG_FLASH_PLAYER = 0,
	AG_QUICK_TIME_PLAYER,
	AG_WINDOWS_MEDIA_PLAYER,
	AG_REAL_PLAYER
} AGPlayer;

typedef struct _AGFontData
{
    float		ascent;
    float		descent;
    float		lineGap;
    float		xHeight;
	AGPitch_t	pitch;
} AGFontData_t;

typedef struct _AGPath
{
    void* data;
} AGPath_t;

typedef struct _AGScaledFont
{
	void*	data;
	unsigned int refCount;
} AGScaledFont_t;

typedef struct _AGGlyph
{
	unsigned long	index;
	UChar			unicodeChar;
	double			x;
	double			y;
} AGGlyph_t;

typedef struct _AGParams
{
	char* name;
	char* value;
} AGParams_t;

typedef struct _AGRawData
{
	const char* data;
	unsigned int size;
} AGRawData_t;

typedef enum _AGImageFormat
{
	AG_IMAGE_FORMAT_ARGB32
} AGImageFormat_t;


typedef enum _AGEmbedMultimedia
{
	MM_NONE,			// Don't embed or reference multimedia content.
	MM_EMBED,			// Embed multimedia content when possible.
	MM_REF,
} AGEmbedMultimedia_t;

typedef enum _AGLineJoin
{
	AG_MITER_JOIN,
	AG_ROUND_JOIN,
	AG_BEVEL_JOIN
} AGLineJoin;

typedef enum _AGLineCap
{
	AG_BUTT_CAP,
	AG_ROUND_CAP,
	AG_SQUARE_CAP
} AGLineCap;

typedef enum _AGResourceType
{
	AG_INVALID_RESOURCE = 0
	, AG_RESOURCE_IMAGE
	, AG_RESOURCE_CSS_STYLE_SHEET
	, AG_RESOURCE_SCRIPT
	, AG_RESOURCE_EMBED_OBJECT
	, AG_RESOURCE_HTML
} AGResourceType_t;

typedef void (ACCB2 * AGImageMemBufDeleteCallBackPtr)(void *);

typedef struct _AGImage 
{
	AGImageFormat_t format;
    unsigned char *rgbaData;
	int rgbaDataSize;
	float x;
	float y;
	float imageWidth;
    float imageHeight;
	float srcWidth;
    float srcHeight;
	float tileWidth;
	float tileHeight;
	float destWidth;
	float destHeight;
    int depth;
	int offsetX;
	int offsetY;
	bool isTiled;
	bool hasAlpha;

	/* This is not owned by AGImage.
	1. Creator has the responsibility of ensuring that the lifetime of AGImage_t
		is within the lifetime of the platform_data.
	2. AGImage_t and its users, on their part, guarentee not to hold references
		to this platform data.
	*/
	WebKitAGPlatformData platform_data;

	AGImageMemBufDeleteCallBackPtr image_del_cb_func;

} AGImage_t;

typedef struct _AGMatrix
{
	double a, b, c, d, h, v;
} AGMatrix_t;

typedef struct _AGPageMargin
{
	float left;
	float right;
	float top;
	float bottom;
} AGPageMargin_t;

enum FormFieldType
{
	ftNone,
	ftButton,
	ftMultilineText,
	ftTextbox,
	ftPassword,
	ftFileSelect,
	ftDropdown,
	ftMultiselectList,
	ftCheckbox,
	ftRadioButton,
	ftSubmit,
	ftImage,
	ftReset,
	ftHidden
};

typedef struct _AGTextToValueMapStruct
{
	const UChar* text;
	const UChar* value;
} AGTextToValueMapStruct;

typedef enum _AGMethod
{
	methodGET,
	methodPOST
} AGMethod;

enum FormFieldAttributes
{
	attrNone = 0,
	attrReadOnly = 0x1,
	attrDisabled = 0x2,
	attrHasBackgroundFill = 0x4,
	attrHasBackgroundImage = 0x8,
	attrHasBorder = 0x10,
	attrIsChecked = 0x20,
};

enum FormTextAttributes
{
	attrTextNone = 0,
	attrTextItalicized = 0x1
};

typedef struct _AGBackgroundImageAttrs
{
	int xOffset;
	int yOffset;
	bool bRepeatX;
	bool bRepeatY;
} AGBackgroundImageAttrs;

typedef struct _AGFieldSubmitDetailsStruct
{
	const UChar** fieldNames;
	const UChar* submitURL;
	AGMethod method;
} AGFieldSubmitDetailsStruct;

typedef struct _AGFormFieldDetails
{
	FormFieldType ffType;
	const UChar* fieldName;
	const UChar* fieldValue;
	const UChar* formName;
	const UChar* baseName;
	FormFieldAttributes attributes;
	FormTextAttributes textAttributes;

	AGTextToValueMapStruct** textToValueMap;
	AGFieldSubmitDetailsStruct* fieldSubmitDetails;
	
	int borderColor[4];
	int bgColor[4];
	int fontSize;
	int fontWeight;
	int textColor[4];
	const UChar* fontName;
	AGImage_t* image;
	AGBackgroundImageAttrs* backgroundImageAttrs;
} AGFormFieldDetails;

typedef struct _AGMapAreaData
{
	int x;
	int y;
	int width;
	int height;
	bool isDefault;
	char* href;
	char* nodeName;
	char* path;
	char* unique_path;
} AGMapAreaData;

typedef enum _AGTagState
{
	AG_START_TAG,
	AG_END_TAG
} AGTagState;

typedef enum _AGPaintPhase
{
	AG_PAINT_FOREGROUND,
	AG_PAINT_BACKGROUND
} AGPaintPhase;

typedef struct _AGHTMLTagData
{
	AGTagState tagState;
	AGPaintPhase paintPhase;
	bool	isLink;
	bool	isMap;
	bool	createTag;
	int		numMapAreaItems;
	int		x;
	int		y;
	int		width;
	int		height;
	bool	isNamedDestination;
	char*	path;
	char*   unique_path;
	char*	href;
	char*	nodeName;
	char*   base_url;
	char*	dest_name;
	char*	dest_id;
	int     isMultiMedia;
	char*   serviceType;
	char*	multiMediaURL;
	AGParams_t* params;
	int param_size;
	AGRawData_t contentData;
	int		isFormField;
	AGFormFieldDetails* formFieldDetails;
	AGMapAreaData* mapAreaData;
} AGHTMLTagData_t;

typedef struct _AGFontBackend AGFontBackend_t;
typedef struct _AGContext AGContext_t;
typedef void* AGFontServer_t;

/* Note: AGEncoding_t, AGFont_t and AGScaledFont_t are created with an initial refcount of 1 */
struct _AGFontBackend
{
	AGFontServer_t ag_fontserver;
	ACCB1 AGStatus_t (ACCB2 *agInitFontServer)(AGFontBackend_t* ag_fontbackend);
	ACCB1 AGFont_t* (ACCB2 *ag_create_font)(AGFontBackend_t* ag_fontbackend, unsigned short* fontName, unsigned nameLen, unsigned weight, bool italic, unsigned width);
	ACCB1 AGFont_t* (ACCB2 *ag_create_font_from_platformdata)(AGFontBackend_t* ag_fontbackend, void* platformData, int size);
	ACCB1 const char* (ACCB2 *ag_get_best_font_and_encoding_for_character)(const UChar c, AGFont_t* agFont, WebKitAGLanguageScript langScript, AGEncodingType_t* encoding);
	ACCB1 AGEncoding_t* (ACCB2 *ag_create_encoding)(AGFontBackend_t* ag_fontbackend, AGEncodingType_t encodingType);
	ACCB1 AGEncoding_t* (ACCB2 *ag_create_encoding_from_codepage)(AGFontBackend_t* ag_fontbackend, const UChar* codepage, int encodingLen);
	ACCB1 AGEncoding_t* (ACCB2 *ag_create_encoding_from_cmap_name)(AGFontBackend_t* ag_fontbackend, const char* cmapName, int cmapNameLen);
	ACCB1 AGScaledFont_t* (ACCB2 *ag_scaled_font_create)(AGFontBackend_t* ag_fontbackend, AGFont_t* ag_font, AGEncoding_t* ag_encoding, AGMatrix_t* sizeMatrix, AGMatrix_t* ctm);
	ACCB1 AGFont_t* (ACCB2 *ag_get_font_from_scaled_font)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* ag_scaled_font);
	ACCB1 AGStatus_t (ACCB2 *ag_set_font_face)(AGFontBackend_t* ag_fontbackend, char* fontface);
	ACCB1 AGStatus_t (ACCB2 *ag_set_font_size)(AGFontBackend_t* ag_fontbackend, int size);
	ACCB1 AGStatus_t (ACCB2 *ag_scaled_font_select_font)(AGContext_t* ag_context, AGScaledFont_t* scaledFont);
	ACCB1 AGStatus_t (ACCB2 *ag_scaled_font_done_font)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* scaledFont);
	ACCB1 AGFontData_t (ACCB2 *ag_get_font_data)(AGFontBackend_t* ag_fontbackend, AGFont_t* ag_font);
	ACCB1 int (ACCB2 *ag_get_num_glyph_ids)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* ag_scaled_font, UChar* characters, int l);
	ACCB1 AGStatus_t (ACCB2 *ag_get_glyph_ids)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* ag_scaled_font, UChar* characters, int l, UChar* glyphBuf);
	ACCB1 float (ACCB2 *ag_get_width)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* ag_scaled_font, UChar glyph);
	ACCB1 float (ACCB2 *ag_get_glyph_width)(AGFontBackend_t* ag_fontbackend, AGScaledFont_t* ag_scaled_font, UChar glyphID);
	ACCB1 AGStatus_t (ACCB2 *ag_terminate_font_server)(AGFontBackend_t* ag_fontbackend);
	ACCB1 AGStatus_t (ACCB2 *ag_encoding_addref)(AGEncoding_t* ag_encoding);
	ACCB1 AGStatus_t (ACCB2 *ag_encoding_release)(AGEncoding_t* ag_encoding);
	ACCB1 AGStatus_t (ACCB2 *ag_font_addref)(AGFont_t* ag_font);
	ACCB1 AGStatus_t (ACCB2 *ag_font_release)(AGFont_t* ag_font);
	ACCB1 AGStatus_t (ACCB2 *ag_scaled_font_addref)(AGScaledFont_t* ag_scaled_font);
	ACCB1 AGStatus_t (ACCB2 *ag_scaled_font_release)(AGScaledFont_t* ag_scaled_font);
};


typedef struct _AGBackend
{
	ACCB1 AGStatus_t	(ACCB2 *ag_save)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_restore)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_color_rgba)(AGContext_t* ag_context, float red, float green, float blue, float alpha);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_transparency)(AGContext_t* ag_context, float opacity);
	ACCB1 AGStatus_t	(ACCB2 *ag_end_transparency)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_alpha)(AGContext_t* ag_context, float opacity);
	ACCB1 AGStatus_t    (ACCB2 *ag_set_line_join)(AGContext_t* ag_context, AGLineJoin lineJoin);
	ACCB1 AGStatus_t    (ACCB2 *ag_set_line_cap)(AGContext_t* ag_context, AGLineCap lineCap);
	ACCB1 AGStatus_t    (ACCB2 *ag_set_miter_limit)(AGContext_t* ag_context, float miterLimit);
	ACCB1 AGStatus_t	(ACCB2 *ag_moveto)(AGContext_t* ag_context, float x, float y);
	ACCB1 AGStatus_t	(ACCB2 *ag_lineto)(AGContext_t* ag_context, float x, float y);
	ACCB1 AGStatus_t	(ACCB2 *ag_rect)(AGContext_t* ag_context, float x, float y, float w, float h);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_line_width)(AGContext_t* ag_context, float lw);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_line_dash)(AGContext_t* ag_context, float offset, int num_dashes, float *dashArray);
	ACCB1 AGStatus_t	(ACCB2 *ag_stroke)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_fill)(AGContext_t* ag_context, AGFillRule_t fill_rule);
	ACCB1 AGStatus_t	(ACCB2 *ag_clip)(AGContext_t* ag_context, float x, float y, float width, float height);
	ACCB1 AGStatus_t	(ACCB2 *ag_EOClip)(AGContext_t* ag_context, bool isEOClip);
	ACCB1 AGStatus_t	(ACCB2 *ag_begin_path)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_close_path)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_arc)(AGContext_t* ag_context, float xc, float yc, float xRadius, float yRadius, float startAngle, float endAngle, bool useMoveTo);
	ACCB1 AGStatus_t	(ACCB2 *ag_roundedRect_clip)(AGContext_t* ag_context, float x, float y, float w, float h, float topLeftRadius, float topRightRadius, float bottomLeftRadius, float bottomRightRadius);
	ACCB1 AGStatus_t	(ACCB2 *ag_inner_roundedRect_clip)(AGContext_t* ag_context, float x, float y, float w, float h, int thickness);
	ACCB1 AGStatus_t	(ACCB2 *ag_get_CTM)(AGContext_t* ag_context, AGMatrix_t *ag_matrix);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_CTM)(AGContext_t* ag_context, AGMatrix_t *ag_matrix);
	ACCB1 AGStatus_t	(ACCB2 *ag_show_glyphs)(AGContext_t* ag_context, AGScaledFont_t* ag_font, AGGlyph_t* glyphs, int numGlyphs, AGEncoding_t* ag_encoding, AGMatrix_t* sizeMatrix, AGMatrix_t* ctm);
	ACCB1 float			(ACCB2 *ag_get_surface_width)(AGContext_t* ag_context);
	ACCB1 float			(ACCB2 *ag_get_surface_height)(AGContext_t* ag_context);
	ACCB1 int			(ACCB2 *ag_get_overlap_height_prev_page)(AGContext_t* ag_context);
	ACCB1 AGStatus_t	(ACCB2 *ag_set_overlap_height_next_page)(AGContext_t* ag_context, int height);
	ACCB1 AGStatus_t    (ACCB2 *ag_set_curr_bottom_line)(AGContext_t* ag_context, int height);
	ACCB1 int		    (ACCB2 *ag_get_prev_bottom_line)(AGContext_t* ag_context);
	ACCB1 int		    (ACCB2 *ag_get_curr_bottom_line)(AGContext_t* ag_context);		
	ACCB1 int		    (ACCB2 *ag_get_scaled_content_height)(AGContext_t* ag_context);		
	ACCB1 AGStatus_t    (ACCB2 *ag_set_scaled_content_height)(AGContext_t* ag_context, int height);
	ACCB1 int		    (ACCB2 *ag_get_frame_offset)(AGContext_t* ag_context);		
	ACCB1 AGStatus_t    (ACCB2 *ag_set_frame_offset)(AGContext_t* ag_context, int offset);
	ACCB1 AGStatus_t	(ACCB2 *ag_get_page_margin)(AGContext_t* ag_context, AGPageMargin_t* pageMargin);
	ACCB1 AGStatus_t	(ACCB2 *ag_start_current_tag_info)(AGContext_t* ag_context, AGHTMLTagData_t* html_tag_info);
	ACCB1 AGStatus_t	(ACCB2 *ag_stop_current_tag_info)(AGContext_t* ag_context, AGHTMLTagData_t* html_tag_info);
	ACCB1 AGStatus_t	(ACCB2 *ag_draw_image)(AGContext_t* ag_context, AGImage_t* ag_Image);
} AGBackend_t;

struct _AGContext
{
	void*				ag_surface;
	const AGBackend_t*	ag_backend;
	AGFontBackend_t*	ag_fontbackend;
};

#endif /* __AG_H_ */
