/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2007 Trolltech ASA
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "MIMETypeRegistry.h"

#include "WebCoreSystemInterface.h"

namespace WebCore 
{
String getMIMETypeForUTI(const String & uti)
{
    CFStringRef utiref = uti.createCFString();
    CFStringRef mime = UTTypeCopyPreferredTagWithClass(utiref, kUTTagClassMIMEType);
    String mimeType = mime;
    if (mime)
        CFRelease(mime);
    CFRelease(utiref);
    return mimeType;
}

String MIMETypeRegistry::getMIMETypeForExtension(const String &ext)
{
#if PLATFORM(AG)
    if (ext.isEmpty())
        return String();

    static HashMap<String, String> mimetypeMap;
    if (mimetypeMap.isEmpty()) {
        //fill with initial values
        mimetypeMap.add("txt", "text/plain");
        mimetypeMap.add("pdf", "application/pdf");
        mimetypeMap.add("ps", "application/postscript");
        mimetypeMap.add("html", "text/html");
        mimetypeMap.add("htm", "text/html");
        mimetypeMap.add("xml", "text/xml");
        mimetypeMap.add("xsl", "text/xsl");
        mimetypeMap.add("js", "application/x-javascript");
        mimetypeMap.add("xhtml", "application/xhtml+xml");
        mimetypeMap.add("rss", "application/rss+xml");
        mimetypeMap.add("webarchive", "application/x-webarchive");
        mimetypeMap.add("svg", "image/svg+xml");
        mimetypeMap.add("svgz", "image/svg+xml");
        mimetypeMap.add("jpg", "image/jpeg");
        mimetypeMap.add("jpeg", "image/jpeg");
        mimetypeMap.add("png", "image/png");
        mimetypeMap.add("tif", "image/tiff");
        mimetypeMap.add("tiff", "image/tiff");
        mimetypeMap.add("ico", "image/ico");
        mimetypeMap.add("cur", "image/ico");
        mimetypeMap.add("bmp", "image/bmp");
    }
	return mimetypeMap.get(ext);
#else
    return wkGetMIMETypeForExtension(ext);
#endif
}

Vector<String> MIMETypeRegistry::getExtensionsForMIMEType(const String& type)
{
    NSArray *stringsArray = wkGetExtensionsForMIMEType(type);
    Vector<String> stringsVector = Vector<String>();
    unsigned count = [stringsArray count];
    if (count > 0) {
        NSEnumerator* enumerator = [stringsArray objectEnumerator];
        NSString* string;
        while ((string = [enumerator nextObject]) != nil)
            stringsVector.append(string);
    }
    return stringsVector;
}

String MIMETypeRegistry::getPreferredExtensionForMIMEType(const String& type)
{
#if PLATFORM(AG)
    if (type.isEmpty())
        return String();

    static HashMap<String, String> mimetypeMap;
    if (mimetypeMap.isEmpty()) {
        // fill with initial values
        mimetypeMap.add("text/plain", "txt");
        mimetypeMap.add("application/pdf", "pdf");
        mimetypeMap.add("application/postscript", "ps");
        mimetypeMap.add("text/html", "html");
        mimetypeMap.add("text/xml", "xml");
        mimetypeMap.add("text/xsl", "xsl");
        mimetypeMap.add("application/x-javascript", "js");
        mimetypeMap.add("application/xhtml+xml", "xhtml");
        mimetypeMap.add("application/rss+xml", "xml");
        mimetypeMap.add("image/svg+xml", "svg");
        mimetypeMap.add("image/jpeg", "jpg");
        mimetypeMap.add("image/png", "png");
        mimetypeMap.add("image/tiff", "tif");
        mimetypeMap.add("image/tiff", "tiff");
        mimetypeMap.add("image/ico", "ico");
        mimetypeMap.add("image/bmp", "bmp");
    }
    String result = mimetypeMap.get(type);
    return result;
#else
    return wkGetPreferredExtensionForMIMEType(type);
#endif
}

}
